/*
 * menusched.c: EPG2VDR plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 */

#define __STL_CONFIG_H

#include <vdr/menuitems.h>
#include <vdr/status.h>
#include <vdr/menu.h>

#include "plgconfig.h"
#include "menu.h"

#include "ttools.h"

#if defined (APIVERSNUM) && (APIVERSNUM >= 20301)
#  warning "Program menu have to be ported to VDR >= 2.3.1, compiling without it"
#else

#define NEWTIMERLIMIT    60 // seconds until the start time of a new timer created from the Schedule menu,
                            // within which it will go directly into the "Edit timer" menu to allow
                            // further parameter settings

#define MAXCHNAMWIDTH    16 // maximum number of characters of channels' short names shown in schedules menus

#define CHNUMWIDTH       (numdigits(Channels.MaxNumber()) + 1)
#define CHNAMWIDTH       (std::min(MAXCHNAMWIDTH, Channels.MaxShortChannelNameLength() + 1))

//***************************************************************************
//***************************************************************************
// Class cMenuEpgMatchRecordings
//***************************************************************************

class cMenuEpgMatchRecordings : public cOsdMenu 
{
   public:

      cMenuEpgMatchRecordings(cMenuDb* db, const cEvent* event);
      virtual ~cMenuEpgMatchRecordings() {};
      virtual eOSState ProcessKey(eKeys Key);
};

cMenuEpgMatchRecordings::cMenuEpgMatchRecordings(cMenuDb* db, const cEvent* event)
   : cOsdMenu(tr("Matching recordings"), 30, 30, 30, 40)
{
   cMenuDb* menuDb = db;
   
   Add(new cOsdItem(tr("Direct Matches:")));
   cList<cOsdItem>::Last()->SetSelectable(no);
  
   tell(0, "Search recordings for '%s' '%s'",
        menuDb->recordingListDb->getStrValue("TITLE"),
        menuDb->recordingListDb->getStrValue("SHORTTEXT"));

   if (!isEmpty(event->ShortText()))
   {
      menuDb->recordingListDb->clear();
      menuDb->recordingListDb->setValue("TITLE", event->Title());
      menuDb->recordingListDb->setValue("SHORTTEXT", event->ShortText());
      
      for (int f = menuDb->selectRecordingForEvent->find(); f; f = menuDb->selectRecordingForEvent->fetch())
      {
         const char* vdr = 0;

#ifdef WITH_PIN     
         if (!cOsd::pinValid && menuDb->recordingListDb->getIntValue("FSK"))
            continue;
#endif

         if (!menuDb->recordingListDb->getValue("OWNER")->isEmpty())
         {
            menuDb->vdrDb->clear();
            menuDb->vdrDb->setValue("UUID", menuDb->recordingListDb->getStrValue("OWNER"));
            
            if (menuDb->vdrDb->find())
               vdr = menuDb->vdrDb->getStrValue("NAME");
         }

         Add(new cEpgMenuTextItem(menuDb->recordingListDb->getStrValue("MD5PATH"), 
                                  cString::sprintf("%s%s%s\t%s\t%s/%s", vdr ? vdr : "", vdr ? "\t" : "", 
                                                   menuDb->recordingListDb->getStrValue("TITLE"),
                                                   menuDb->recordingListDb->getStrValue("SHORTTEXT"),
                                                   menuDb->recordingListDb->getStrValue("FOLDER"),
                                                   menuDb->recordingListDb->getStrValue("NAME"))));
      }
      
      menuDb->selectRecordingForEvent->freeResult();
   }

   Add(new cOsdItem(tr("All Matches:")));
   cList<cOsdItem>::Last()->SetSelectable(no);

   menuDb->recordingListDb->clear();
   menuDb->recordingListDb->setValue("TITLE", event->Title());
   menuDb->recordingListDb->setValue("SHORTTEXT", "%");

   for (int f = menuDb->selectRecordingForEvent->find(); f; f = menuDb->selectRecordingForEvent->fetch())
   {
      const char* vdr = 0;
      
#ifdef WITH_PIN     
         if (!cOsd::pinValid && menuDb->recordingListDb->getIntValue("FSK"))
            continue;
#endif

      if (!menuDb->recordingListDb->getValue("OWNER")->isEmpty())
      {
         menuDb->vdrDb->clear();
         menuDb->vdrDb->setValue("UUID", menuDb->recordingListDb->getStrValue("OWNER"));
         
         if (menuDb->vdrDb->find())
            vdr = menuDb->vdrDb->getStrValue("NAME");
      }

      Add(new cEpgMenuTextItem(menuDb->recordingListDb->getStrValue("MD5PATH"), 
                               cString::sprintf("%s%s%s\t%s\t%s/%s", vdr ? vdr : "", vdr ? "\t" : "",
                                                menuDb->recordingListDb->getStrValue("TITLE"),
                                                menuDb->recordingListDb->getStrValue("SHORTTEXT"),
                                                menuDb->recordingListDb->getStrValue("FOLDER"),
                                                menuDb->recordingListDb->getStrValue("NAME"))));
   }
   
   menuDb->selectRecordingForEvent->freeResult();

   Display();
}

eOSState cMenuEpgMatchRecordings::ProcessKey(eKeys Key)
{
   eOSState state = cOsdMenu::ProcessKey(Key);
   
   if (state == osUnknown) 
   {
      return osContinue;
   }

   return state;
}

//***************************************************************************
//***************************************************************************
// Class cMenuEpgScheduleItem
//***************************************************************************

class cMenuEpgScheduleItem : public cOsdItem 
{
   public:

      enum eScheduleSortMode { ssmAllThis, ssmThisThis, ssmThisAll, ssmAllAll }; // "which event(s) on which channel(s)"
      
      cMenuEpgScheduleItem(cMenuDb* db, const cEvent* Event, cChannel* Channel = 0, bool WithDate = no);

      static void SetSortMode(eScheduleSortMode SortMode)  { sortMode = SortMode; }
      static void IncSortMode()                            { sortMode = eScheduleSortMode((sortMode == ssmAllAll) ? ssmAllThis : sortMode + 1); }
      static eScheduleSortMode SortMode()                  { return sortMode; }

      virtual int Compare(const cListObject &ListObject) const;
      virtual bool Update(bool Force = false);
      virtual void SetMenuItem(cSkinDisplayMenu *DisplayMenu, int Index, bool Current, bool Selectable);

      const cEvent* event;
      const cChannel* channel;
      bool withDate;
      int timerMatch;
      
   private:

      cMenuDb* menuDb;
      static eScheduleSortMode sortMode;
};

//***************************************************************************
// 
//***************************************************************************

cMenuEpgScheduleItem::eScheduleSortMode cMenuEpgScheduleItem::sortMode = ssmAllThis;

//***************************************************************************
// Object
//***************************************************************************

cMenuEpgScheduleItem::cMenuEpgScheduleItem(cMenuDb* db, const cEvent* Event, cChannel* Channel, bool WithDate)
{
   menuDb = db;

   event = Event;
   channel = Channel;
   withDate = WithDate;
   timerMatch = tmNone;
   Update(true);
}

//***************************************************************************
// Compare
//***************************************************************************

int cMenuEpgScheduleItem::Compare(const cListObject &ListObject) const
{
   cMenuEpgScheduleItem *p = (cMenuEpgScheduleItem *)&ListObject;
   int r = -1;

   if (sortMode != ssmAllThis)
      r = strcoll(event->Title(), p->event->Title());

   if (sortMode == ssmAllThis || r == 0)
      r = event->StartTime() - p->event->StartTime();

   return r;
}

//***************************************************************************
// Update
//***************************************************************************

bool cMenuEpgScheduleItem::Update(bool Force)
{
   static const char* TimerMatchChars = " tT";
   static const char* TimerMatchCharsRemote = " sS";

   bool result = false;
   int oldTimerMatch = timerMatch;
   int remote = no;

   menuDb->lookupTimer(event, timerMatch, remote); // -> loads timerDb and vdrDb

   if (Force || timerMatch != oldTimerMatch) 
   {
      cString buffer;
      char t = !remote ? TimerMatchChars[timerMatch] : TimerMatchCharsRemote[timerMatch];
      char v = event->Vps() && (event->Vps() - event->StartTime()) ? 'V' : ' ';
      char r = event->SeenWithin(30) && event->IsRunning() ? '*' : ' ';
      const char* csn = channel ? channel->ShortName(true) : 0;
      cString eds = event->GetDateString();

      if (channel && withDate)
         buffer = cString::sprintf("%d\t%.*s\t%.*s\t%s\t%c%c%c\t%s", channel->Number(), 
                                   Utf8SymChars(csn, 999), csn, Utf8SymChars(eds, 6), 
                                   *eds, *event->GetTimeString(), t, v, r, event->Title());
      else if (channel)
         buffer = cString::sprintf("%d\t%.*s\t%s\t%c%c%c\t%s", channel->Number(), 
                                   Utf8SymChars(csn, 999), csn, *event->GetTimeString(), t, v, r, event->Title());
      else
         buffer = cString::sprintf("%.*s\t%s\t%c%c%c\t%s", Utf8SymChars(eds, 6), *eds, *
                                   event->GetTimeString(), t, v, r, event->Title());

      SetText(buffer);
      result = true;
   }

   return result;
}

//***************************************************************************
// SetMenuItem
//***************************************************************************

void cMenuEpgScheduleItem::SetMenuItem(cSkinDisplayMenu* DisplayMenu, 
                                       int Index, bool Current, bool Selectable)
{
   if (!DisplayMenu->SetItemEvent(event, Index, Current, Selectable, channel, 
                                  withDate, (eTimerMatch)timerMatch))
   {
      DisplayMenu->SetItem(Text(), Index, Current, Selectable);
   }
}

//***************************************************************************
//***************************************************************************
// cMenuMyScheduleSepItem
//***************************************************************************

class cMenuEpgScheduleSepItem : public cOsdItem
{
   public:
      
      cMenuEpgScheduleSepItem(cChannel* Channel, const cEvent* Event);
      ~cMenuEpgScheduleSepItem();

      virtual bool Update(bool Force = false);
      virtual void SetMenuItem(cSkinDisplayMenu *DisplayMenu, int Index, bool Current, bool Selectable);

   private:

      const cChannel* channel;
      const cEvent* event;
      cEvent* tmpEvent;
};

cMenuEpgScheduleSepItem::cMenuEpgScheduleSepItem(cChannel* Channel, const cEvent* Event)
{
   channel = Channel;
   event = Event;
   tmpEvent = 0;

   SetSelectable(false);
   Update(true);
}

cMenuEpgScheduleSepItem::~cMenuEpgScheduleSepItem()
{
   delete tmpEvent;
}

bool cMenuEpgScheduleSepItem::Update(bool Force)
{
   if (channel)
   {
      SetText(cString::sprintf("-----\t %s -----", channel ? channel->Name() : *event->GetDateString()));
   }
   else if (event)
   {
      tmpEvent = new cEvent(0);
      tmpEvent->SetTitle(cString::sprintf("-----\t %s -----", *event->GetDateString()));
      SetText(tmpEvent->Title());
   }

   return true;
}

void cMenuEpgScheduleSepItem::SetMenuItem(cSkinDisplayMenu *DisplayMenu, int Index, bool Current, bool Selectable)
{
   if (!DisplayMenu->SetItemEvent(tmpEvent, Index, Current, Selectable, channel, no, tmNone))
      DisplayMenu->SetItem(Text(), Index, Current, Selectable);
}

//***************************************************************************
//***************************************************************************
// Statics
//***************************************************************************

int cMenuEpgWhatsOn::currentChannel = 0;
const cEvent* cMenuEpgWhatsOn::scheduleEvent = 0;

//***************************************************************************
// Object
//***************************************************************************

cMenuEpgWhatsOn::cMenuEpgWhatsOn(const cEvent* aSearchEvent)
   : cOsdMenu("", CHNUMWIDTH, CHNAMWIDTH, 6, 4)
{
   dispSchedule = no;
   canSwitch = no;
   helpKeys = 0;
   helpKeyTime = 0;
   helpKeyTimeMode = na;
   timerState = 0;
   searchEvent = aSearchEvent;

   menuDb = new cMenuDb;

   cChannel* channel = Channels.GetByNumber(cDevice::CurrentChannel());

   if (channel) 
   {
      currentChannel = channel->Number();
      schedules = cSchedules::Schedules(schedulesLock);
   }

   Timers.Modified(timerState);
   
   menuDb->userTimes->first();     // 'whats on now' should be the first
   
   if (searchEvent)
      LoadQuery(searchEvent);
   else if (menuDb->startWithSched)
      LoadSchedule();
   else
      LoadAt();
}

cMenuEpgWhatsOn::~cMenuEpgWhatsOn()
{
   delete menuDb;
}

//***************************************************************************
// Load At
//***************************************************************************

int cMenuEpgWhatsOn::LoadAt()
{
   dispSchedule = no;
   cUserTimes::UserTime* userTime = menuDb->userTimes->current();
   cMenuEpgScheduleItem* item = (cMenuEpgScheduleItem*)Get(Current());

   if (item)
   {
      scheduleEvent = item->event;
      
      if (item->channel)
         currentChannel = item->channel->Number();
   }
  
   Clear();
   Timers.Modified(timerState);
   SetMenuCategory(userTime->getMode() == cUserTimes::mNow ? mcScheduleNow : mcScheduleNext);

   if (isEmpty(userTime->getHHMMStr()))
      SetTitle(cString::sprintf(tr("Overview - %s"), userTime->getTitle()));
   else
      SetTitle(cString::sprintf(tr("Overview - %s (%s)"), userTime->getTitle(), userTime->getHHMMStr()));

   tell(2, "DEBUG: Loading events for '%s' %s", userTime->getTitle(), !isEmpty(userTime->getHHMMStr()) ? userTime->getHHMMStr() : "");

   // events

   for (cChannel* Channel = Channels.First(); Channel; Channel = Channels.Next(Channel)) 
   {
      if (!Channel->GroupSep()) 
      {
         const cSchedule* Schedule = schedules->GetSchedule(Channel);

         if (Schedule) 
         {
            const cEvent* Event = 0;

            switch (userTime->getMode())
            {
               case cUserTimes::mNow:  Event = Schedule->GetPresentEvent();         break;
               case cUserTimes::mNext: Event = Schedule->GetFollowingEvent();       break;
               case cUserTimes::mTime: Event = Schedule->GetEventAround(userTime->getTime());  break;
            }

            // #TODO
            // hier ein cEpgEvent für das Event aus der Row erzeugen und dieses anstelle von cEvent für das Menü verwenden
            
            if (Event)
               Add(new cMenuEpgScheduleItem(menuDb, Event, Channel), Channel->Number() == currentChannel);
         }
      }
      else
      {
         if (strlen(Channel->Name()))
            Add(new cMenuEpgScheduleSepItem(Channel, 0));
      }
   }

   Display();
   SetHelpKeys();

   return done;
}

int cMenuEpgWhatsOn::LoadSchedule()
{
   Clear();
   SetCols(7, 6, 4);

   dispSchedule = yes;
   cMenuEpgScheduleItem::SetSortMode(cMenuEpgScheduleItem::ssmAllThis);
   cChannel* channel = Channels.GetByNumber(currentChannel);
   SetMenuCategory(mcSchedule);
   SetTitle(cString::sprintf(tr("Schedule - %s"), channel->Name()));

   if (schedules && channel) 
   {
      const cSchedule* Schedule = schedules->GetSchedule(channel);

      if (Schedule) 
      {
         int lastDay = 0;
         const cEvent* PresentEvent = scheduleEvent ? scheduleEvent : Schedule->GetPresentEvent();
         time_t now = time(0) - Setup.EPGLinger * 60;

         for (const cEvent* ev = Schedule->Events()->First(); ev; ev = Schedule->Events()->Next(ev)) 
         {
            time_t stime = ev->StartTime();
            struct tm tmSTime;
            localtime_r(&stime, &tmSTime);
            
            if (lastDay != 0 && tmSTime.tm_mday != lastDay)
               Add(new cMenuEpgScheduleSepItem(0, ev));

            if (ev->EndTime() > now || ev == PresentEvent)
               Add(new cMenuEpgScheduleItem(menuDb, ev), ev == PresentEvent);

            lastDay = tmSTime.tm_mday;
         }
      }
   }

   Display();
   SetHelpKeys();

   return done;
}

int cMenuEpgWhatsOn::LoadQuery(const cEvent* searchEvent)
{
   Clear();
   SetMenuCategory(mcSchedule);
   cMenuEpgScheduleItem::SetSortMode(cMenuEpgScheduleItem::ssmAllThis);

   for (cChannel* channel = Channels.First(); channel; channel = Channels.Next(channel)) 
   {      
      const cSchedule* schedule = schedules->GetSchedule(channel);
      
      if (schedule) 
      {
         for (const cEvent* event = schedule->Events()->First(); event; event = schedule->Events()->Next(event))
         {
            if (rep(event->Title(), searchEvent->Title()) == success &&
                event->StartTime() > time(0))
               Add(new cMenuEpgScheduleItem(menuDb, event, channel, yes));
         }
      }
   }

   Sort();
   SetTitle(cString::sprintf("%d %s - %s", Count(), tr("Search results"), searchEvent->Title()));

   Display();
   SetHelpKeys();

   return Count();
}

//***************************************************************************
// Display
//***************************************************************************

void cMenuEpgWhatsOn::Display()
{
   cOsdMenu::Display();

#ifdef WITH_GTFT
   if (Count() > 0) 
   {
      int ni = 0;

      for (cOsdItem *item = First(); item; item = Next(item)) 
         cStatus::MsgOsdEventItem(((cMenuEpgScheduleItem*)item)->event, item->Text(), ni++, Count());
   }
#endif
}

//***************************************************************************
// Update
//***************************************************************************

bool cMenuEpgWhatsOn::Update()
{
   bool result = false;

   for (cOsdItem* item = First(); item; item = Next(item)) 
   {
      if (((cMenuEpgScheduleItem*)item)->Update())
         result = true;
   }

   return result;
}

//***************************************************************************
// SetHelpKeys
//***************************************************************************

void cMenuEpgWhatsOn::SetHelpKeys()
{
   cMenuEpgScheduleItem* item = (cMenuEpgScheduleItem *)Get(Current());
   canSwitch = no;
   int NewHelpKeys = 0;
      
   if (item) 
   {
      if (item->timerMatch == tmFull)
         NewHelpKeys |= 0x02;    // "Timer"
      else
         NewHelpKeys |= 0x01;    // "Record"

      if (!dispSchedule)
         NewHelpKeys |= 0x04;    // "Schedule"
      
      if (cChannel* Channel = Channels.GetByChannelID(item->event->ChannelID(), true)) 
      {
         if (Channel->Number() != cDevice::CurrentChannel()) 
         {
            NewHelpKeys |= 0x10; // "Switch"
            canSwitch = yes;
         }
      }
   }

   cUserTimes::UserTime* userTimeGreen = !dispSchedule ? menuDb->userTimes->getNext() : menuDb->userTimes->current();
   
   if (NewHelpKeys != helpKeys || helpKeyTime != userTimeGreen->getTime() || helpKeyTimeMode != userTimeGreen->getMode())
   {
      const char* Red[] = { 0, tr("Button$Record"), tr("Button$Timer") };

      if (searchEvent)
      {
         helpKeys = 0;
         helpKeyTime = 0;
         helpKeyTimeMode = na;
         
         SetHelp(Red[NewHelpKeys & 0x03], 0, 0, 0);
      }
      else
      {
         SetHelp(Red[NewHelpKeys & 0x03],                                   // red
                 userTimeGreen->getHelpKey(),                               // green
                 !dispSchedule
                 ? tr("Button$Schedule")                                    // yellow
                 : menuDb->userTimes->current() == menuDb->userTimes->getFirst()
                 ? tr(menuDb->userTimes->getNext()->getHelpKey())           // yellow
                 : tr(menuDb->userTimes->getFirst()->getHelpKey()),         // yellow
                 canSwitch ? tr("Button$Switch") : 0);                      // blue

         helpKeyTime = userTimeGreen->getTime();
         helpKeyTimeMode = userTimeGreen->getMode();
         helpKeys = NewHelpKeys;
      }
   }
}

//***************************************************************************
// Switch
//***************************************************************************

eOSState cMenuEpgWhatsOn::Switch()
{
   cMenuEpgScheduleItem* item = (cMenuEpgScheduleItem*)Get(Current());
   
   if (item) 
   {
      cChannel* channel = Channels.GetByChannelID(item->event->ChannelID(), true);

      if (channel && cDevice::PrimaryDevice()->SwitchChannel(channel, true))
         return osEnd;
   }

   Skins.Message(mtError, tr("Can't switch channel!"));

   return osContinue;
}

//***************************************************************************
// Record
//***************************************************************************

eOSState cMenuEpgWhatsOn::Record()
{
   int timerMatch = tmNone;
   cEpgTimer* timer = 0;
   long timerid = 0;
   int remote = no;
   cMenuEpgScheduleItem* item = (cMenuEpgScheduleItem*)Get(Current());

   if (!item)
      return osContinue;

   if (timerid = menuDb->lookupTimer(item->event, timerMatch, remote)) // -> loads timerDb and vdrDb
   {
      menuDb->timerDb->clear();
      menuDb->timerDb->setValue("ID", timerid);

      if (!menuDb->selectTimerById->find())
      {
         tell(0, "Fatal, Can't find timer (%ld)", timerid);
         return osContinue;
      }

      timer = newTimerObjectFromRow(menuDb->timerDb->getRow(), menuDb->vdrDb->getRow());
   }
   
   if (timer)
      return AddSubMenu(new cMenuEpgEditTimer(menuDb, timer));
   
   // neuen Timer anlegen

   cDbRow* timerRow = newRowFromEvent(item->event);
   // timer = newTimerObjectFromRow(timerRow, xxxxx);
   menuDb->createTimer(timerRow, Epg2VdrConfig.uuid);
   delete timerRow;

   // Menü bei 'aktuellem' Event direkt öffen #TODO

   if (timer && timer->Event() && timer->Event()->StartTime() < time(0) + NEWTIMERLIMIT)
      ; // return AddSubMenu(new cMenuEpgEditTimer(menuDb, timer));

   if (HasSubMenu())
      CloseSubMenu();
   
   if (Update())
      Display();
   
   SetHelpKeys();
   
   return osContinue;
}

//***************************************************************************
// ProcessKey
//***************************************************************************

eOSState cMenuEpgWhatsOn::ProcessKey(eKeys Key)
{
   bool HadSubMenu = HasSubMenu();
   eOSState state = cOsdMenu::ProcessKey(Key);
   
   if (state == osUnknown) 
   {
      if (searchEvent && Key != kOk && Key != kRed && Key != kRecord)
         return osContinue;

      switch (Key) 
      {
         case k1:       // search repeat of event
         {
            cMenuEpgScheduleItem* item = (cMenuEpgScheduleItem*)Get(Current());
            
            if (item) 
               return AddSubMenu(new cMenuEpgWhatsOn(item->event));

            break;
         }

         case k2:      // search matching recordings
         {
            cMenuEpgScheduleItem* item = (cMenuEpgScheduleItem*)Get(Current());
            
            if (item)
               return AddSubMenu(new cMenuEpgMatchRecordings(menuDb, item->event));

            break;
         }

         case kRecord:
         case kRed:    
            return Record();

         case kYellow: 
         {
            if (!dispSchedule)
            {
               cMenuEpgScheduleItem* mi = (cMenuEpgScheduleItem*)Get(Current());
               
               if (mi) 
               {
                  scheduleEvent = mi->event;
                  currentChannel = mi->channel->Number();
                  LoadSchedule();
               }
            }
            else
            {
               if (menuDb->userTimes->current() == menuDb->userTimes->getFirst())
                  menuDb->userTimes->next();
               else
                  menuDb->userTimes->first();
               
               LoadAt();
            }
            
            return osContinue;
         }
         case kGreen:  
         {
            if (!dispSchedule)
               menuDb->userTimes->next();
            
            LoadAt();

            return osContinue;
         }

         case kBlue:   
         {
            if (canSwitch)
               return Switch();

            break;
         }

         case kInfo:
         case kOk:
         {
            if (Count())
               return AddSubMenu(new cMenuEvent(((cMenuEpgScheduleItem *)Get(Current()))->event, canSwitch, true));

            break;
         }

         default: 
            break;
      }
   }

   else if (!HasSubMenu())
   {
      if (HadSubMenu && Update())
         Display();

      if (Key != kNone)
         SetHelpKeys();
   }

   return state;
}

/*
oid cMenuEpgSchedule::PrepareScheduleThisThis(const cEvent *Event, const cChannel *Channel)
{
   Clear();
   SetCols(7, 6, 4);
   SetTitle(cString::sprintf(tr("This event - %s"), Channel->Name()));

   if (schedules && Channel && Event) 
   {
      const cSchedule* Schedule = schedules->GetSchedule(Channel);

      if (Schedule) 
      {
         time_t now = time(0) - Setup.EPGLinger * 60;

         for (const cEvent* ev = Schedule->Events()->First(); ev; ev = Schedule->Events()->Next(ev)) 
         {
            if ((ev->EndTime() > now || ev == Event) && !strcmp(ev->Title(), Event->Title()))
               Add(new cMenuEpgScheduleItem(menuDb, ev), ev == Event);
         }
      }
   }
}

oid cMenuEpgSchedule::PrepareScheduleThisAll(const cEvent *Event, const cChannel *Channel)
{
   Clear();
   SetCols(CHNUMWIDTH, CHNAMWIDTH, 7, 6, 4);
   SetTitle(tr("This event - all channels"));

   if (schedules && Event) 
   {
      for (cChannel* ch = Channels.First(); ch; ch = Channels.Next(ch)) 
      {
         const cSchedule* Schedule = schedules->GetSchedule(ch);

         if (Schedule) 
         {
            time_t now = time(0) - Setup.EPGLinger * 60;

            for (const cEvent* ev = Schedule->Events()->First(); ev; ev = Schedule->Events()->Next(ev)) 
            {
               if ((ev->EndTime() > now || ev == Event) && !strcmp(ev->Title(), Event->Title()))
                  Add(new cMenuEpgScheduleItem(menuDb, ev, ch, true), ev == Event && ch == Channel);
            }
         }
      }
   }
}

oid cMenuEpgSchedule::PrepareScheduleAllAll(const cEvent *Event, const cChannel *Channel)
{
   Clear();
   SetCols(CHNUMWIDTH, CHNAMWIDTH, 7, 6, 4);
   SetTitle(tr("All events - all channels"));

   if (schedules) 
   {
      for (cChannel* ch = Channels.First(); ch; ch = Channels.Next(ch)) 
      {
         const cSchedule* Schedule = schedules->GetSchedule(ch);

         if (Schedule) 
         {
            time_t now = time(0) - Setup.EPGLinger * 60;

            for (const cEvent* ev = Schedule->Events()->First(); ev; ev = Schedule->Events()->Next(ev)) 
            {
               if (ev->EndTime() > now || ev == Event)
                  Add(new cMenuEpgScheduleItem(menuDb, ev, ch, true), ev == Event && ch == Channel);
            }
         }
      }
   }
}

OSState cMenuEpgSchedule::Number()
{
   cMenuEpgScheduleItem::IncSortMode();
   cMenuEpgScheduleItem* CurrentItem = (cMenuEpgScheduleItem*)Get(Current());
   const cChannel* Channel = 0;
   const cEvent* Event = 0;

   if (CurrentItem) 
   {
      Event = CurrentItem->event;
      Channel = Channels.GetByChannelID(Event->ChannelID(), true);
   }
   else
      Channel = Channels.GetByNumber(cDevice::CurrentChannel());
   
   switch (cMenuEpgScheduleItem::SortMode()) 
   {
      // case cMenuEpgScheduleItem::ssmAllThis:  PrepareScheduleAllThis(Event, Channel);  break;
      case cMenuEpgScheduleItem::ssmThisThis: PrepareScheduleThisThis(Event, Channel); break;
      case cMenuEpgScheduleItem::ssmThisAll:  PrepareScheduleThisAll(Event, Channel);  break;
      case cMenuEpgScheduleItem::ssmAllAll:   PrepareScheduleAllAll(Event, Channel);   break;

      default: 
         tell(0, "ERROR: Unknown SortMode %d (%s %d)", cMenuEpgScheduleItem::SortMode(), __FUNCTION__, __LINE__);
   }

   CurrentItem = (cMenuEpgScheduleItem*)Get(Current());
   Sort();
   SetCurrent(CurrentItem);
   Display();

   return osContinue;
}
*/

//***************************************************************************

#endif // defined (APIVERSNUM) && (APIVERSNUM >= 20301)
